@@ -1,4 +1,4 @@
-## ---- include = FALSE---------------------------------------------------------
+## ----include = FALSE----------------------------------------------------------
 knitr::opts_chunk$set(
   collapse = FALSE,
   comment = ">"#,
@@ -14,8 +14,8 @@ sfile <- system.file("extdata", "MozParagon.csv", package = "dcifer")
 dsmp <- readDat(sfile, svar = "sampleID", lvar = "locus", avar = "allele")
 str(dsmp, list.len = 2)
 
-## ---- eval = FALSE------------------------------------------------------------
-#  dsmp <- formatDat(dlong, svar = "sampleID", lvar = "locus", avar = "allele")
+## ----eval = FALSE-------------------------------------------------------------
+# dsmp <- formatDat(dlong, svar = "sampleID", lvar = "locus", avar = "allele")
 
 ## -----------------------------------------------------------------------------
 # optionally, extract location information
@@ -34,12 +34,14 @@ str(afreq, list.len = 2)
 afile  <- system.file("extdata", "MozAfreq.csv", package = "dcifer")
 afreq2 <- readAfreq(afile, lvar = "locus", avar = "allele", fvar = "freq") 
 
-## ---- eval = FALSE------------------------------------------------------------
-#  # alternatively, if allele frequencies are provided in an R data frame (aflong):
-#  afreq2 <- formatAfreq(aflong, lvar = "locus", avar = "allele", fvar = "freq")
+## ----eval = FALSE-------------------------------------------------------------
+# # alternatively, if allele frequencies are provided in an R data frame (aflong):
+# afreq2 <- formatAfreq(aflong, lvar = "locus", avar = "allele", fvar = "freq")
 
 ## -----------------------------------------------------------------------------
-dsmp2  <- matchAfreq(dsmp, afreq2)
+da_upd <- matchAfreq(dsmp, afreq2, minfreq = 1/sum(coi)/10)
+dsmp2  <- da_upd$dsmp
+afreq2 <- da_upd$afreq
 
 ## -----------------------------------------------------------------------------
 provinces <- c("Maputo", "Inhambane")
@@ -48,40 +50,38 @@ ord       <- order(factor(meta$province, levels = provinces))
 dsmp <- dsmp[ord]
 coi  <- coi[ ord]
 
-## ---- eval = FALSE------------------------------------------------------------
-#  dres <- ibdDat(dsmp, coi, afreq, pval = TRUE, confint = TRUE, rnull = 0,
-#                 alpha = 0.05, nr = 1e3)
+## ----eval = FALSE-------------------------------------------------------------
+# dres <- ibdDat(dsmp, coi, afreq, pval = TRUE, confint = TRUE, rnull = 0,
+#                alpha = 0.05, nr = 1e3)
 
 ## -----------------------------------------------------------------------------
 dres[9, 5, ]  
 
-## ---- fig.width = 6, fig.height = 6-------------------------------------------
+## ----fig.width = 6, fig.height = 6--------------------------------------------
 par(mar = c(3, 3, 1, 1))
 nsmp  <- length(dsmp)
 atsep <- cumsum(nsite)[-length(nsite)]
 
 # create symmetric matrix
 dmat <- dres[, , "estimate"]
-dmat[upper.tri(dmat)] <- t(dmat)[upper.tri(t(dmat))] 
+dmat <- mixMat(dmat, dmat)
 
 # determine significant, indices for upper triangle
 alpha <- 0.05                          # significance level                    
-isig <- which(dres[, , "p_value"] <= alpha, arr.ind = TRUE)
-col_id <- rep(c("plum4", "lightblue4"), nsite)
-plotRel(dmat, isig = isig[, 2:1], draw_diag = TRUE, alpha = alpha, idlab = TRUE,
-        col_id = col_id)
+sig <- dres[, , "p_value"] <= alpha
+col_id <- rep(c("orchid4", "cadetblue4"), nsite)
+plotRel(dmat, sig = t(sig), draw_diag = TRUE, idlab = TRUE, col_id = col_id)
 abline(v = atsep, h = atsep, col = "gray45", lty = 5)
 
-## ---- fig.height = 3.6--------------------------------------------------------
+## ----fig.height = 3.6---------------------------------------------------------
 par(mfrow = c(1, 2), mar = c(1, 0, 1, 0.2))
 plotRel(dres, draw_diag = TRUE, alpha = alpha)
 mtext("p-values", 3, 0.2)
 # p-values for upper triangle
-pmat <- matrix(NA, length(dsmp), length(dsmp))
-pmat[upper.tri(pmat)] <- t(log(dres[, , "p_value"]))[upper.tri(pmat)]
-pmat[pmat == -Inf] <- min(pmat[is.finite(pmat)])#*1.2
+pmat <- t(log(dres[, , "p_value"]))
+pmat[pmat == -Inf] <- min(pmat[is.finite(pmat)])
 plotRel(pmat, rlim = NULL, draw_diag = TRUE, col = hcl.colors(101, "Red-Purple"), 
-        sig = FALSE, add = TRUE, col_diag = "white", border_diag = "gray45")
+        add = TRUE, col_diag = "white", border_diag = "gray45")
 abline(v = atsep, h = atsep, col = "gray45", lty = 5)
 
 # number of non-missing loci for upper triangle
@@ -92,7 +92,7 @@ for (jsmp in 2:nsmp) {
     nmat[ismp, jsmp] <- sum(!dmiss[[ismp]] & !dmiss[[jsmp]])
   }
 }
-nrng <- range(nmat, na.rm = TRUE)  #  
+nrng <- range(nmat, na.rm = TRUE)    
 par(mar = c(1, 0.2, 1, 0))
 plotRel(dres, draw_diag = TRUE, alpha = alpha)
 mtext("number of loci", 3, 0.2)
@@ -101,10 +101,10 @@ plotRel(nmat, rlim = NA, col = coln, add = TRUE,
         draw_diag = TRUE, col_diag = "gray45", border_diag = "white")
 par(pardef)
 
-## ---- fit.width = 7, fig.height = 5.8-----------------------------------------
+## ----fit.width = 7, fig.height = 5.8------------------------------------------
 layout(matrix(1:2, 1), width = c(7, 1))
-par(mar = c(1, 1, 2, 1))#, mgp = c(0, 0, 0.5))
-plotRel(dmat, draw_diag = TRUE, isig = rbind(isig, isig[, 2:1]))
+par(mar = c(1, 1, 2, 1))
+plotRel(dmat, sig = mixMat(sig, sig), draw_diag = TRUE)
 atclin <- cumsum(nsite) - nsite/2
 abline(v = atsep, h = atsep, col = "gray45", lty = 5)
 mtext(provinces, side = 3, at = atclin, line = 0.2)
@@ -113,10 +113,10 @@ par(mar = c(1, 0, 2, 3))
 plotColorbar()
 par(pardef)
 
-## ---- fig.width = 6.2, fig.height = 6-----------------------------------------
-# horizontal colorbar (needed?)
+## ----fig.width = 6.2, fig.height = 6------------------------------------------
+# horizontal colorbar 
 par(mar = c(1, 1, 1, 3))
-border_sig = "darkviolet"
+border_sig <- "darkviolet"
 plotRel(dres, draw_diag = TRUE, border_diag = border_sig, alpha = alpha, 
         border_sig = border_sig, lwd_sig = 2)
 legend(32, 20, pch = 0, col = border_sig, pt.lwd = 2, pt.cex = 1.4, 
@@ -129,17 +129,19 @@ ncol <- 301
 lines(c(0, ncol, ncol, 0, 0), c(0, 0, 1, 1, 0), col = "gray")
 par(pardef)
 
-## ---- eval = FALSE------------------------------------------------------------
-#  # First, create a grid of r values to evaluate over
-#  revals <- mapply(generateReval, 1:5, nr = c(1e3, 1e2, 32, 16, 12))
+## ----eval = FALSE-------------------------------------------------------------
+# # First, create a grid of r values to evaluate over
+# revals <- mapply(generateReval, 1:5, nr = c(1e3, 1e2, 32, 16, 12))
 
 ## -----------------------------------------------------------------------------
-sig1 <- sig2 <- vector("list", nrow(isig))
-for (i in 1:nrow(isig)) {
+isig <- which(sig, arr.ind = TRUE)
+nsig <- nrow(isig)
+sig1 <- sig2 <- vector("list", nsig)
+for (i in 1:nsig) {
   sig1[[i]] <- ibdEstM(dsmp[isig[i, ]], coi[isig[i, ]], afreq, Mmax = 5, 
                        equalr = FALSE, reval = revals)
 }
-for (i in 1:nrow(isig)) {
+for (i in 1:nsig) {
   sig2[[i]] <- ibdEstM(dsmp[isig[i, ]], coi[isig[i, ]], afreq, equalr = TRUE)
 }
 M1      <- sapply(sig1, function(r) sum(r > 0))  
@@ -152,11 +154,11 @@ cor(rtotal1, rtotal2)
 
 ## -----------------------------------------------------------------------------
 samples <- names(dsmp)
-sig <- as.data.frame(isig, row.names = FALSE)
-sig[c("id1", "id2")]         <- list(samples[isig[, 1]], samples[isig[, 2]])
-sig[c("M1", "M2")]           <- list(M1, M2)
-sig[c("rtotal1", "rtotal2")] <- list(round(rtotal1, 3), round(rtotal2, 3))
-head(sig)
+dfsig <- as.data.frame(isig, row.names = FALSE)
+dfsig[c("id1", "id2")]         <- list(samples[isig[, 1]], samples[isig[, 2]])
+dfsig[c("M1", "M2")]           <- list(M1, M2)
+dfsig[c("rtotal1", "rtotal2")] <- list(round(rtotal1, 3), round(rtotal2, 3))
+head(dfsig)
 
 ## -----------------------------------------------------------------------------
 i <- 18                              
@@ -171,7 +173,7 @@ res2 <- ibdPair(pair, coii, afreq, M = M2[i], pval = TRUE, equalr = TRUE,
 res1$rhat                              # estimate with equalr = FALSE
 rep(res2$rhat, M2[i])                  # estimate with equalr = TRUE
 
-## ---- fig.width = 6, fig.height = 4.5-----------------------------------------
+## ----fig.width = 6, fig.height = 4.5------------------------------------------
 CI     <- range(res2$confreg)
 llikCI <- max(res2$llik) - qchisq(1 - alpha, df = 1)/2
 llrng  <- range(res2$llik[is.finite(res2$llik)])
@@ -181,7 +183,7 @@ cols <- c("purple", "cadetblue3", "gray60")
 
 par(mar = c(3, 2.5, 2, 0.1), mgp = c(1.5, 0.3, 0))
 plot(revals[[1]], res2$llik, type = "l", xlab = "r", ylab = "log-likelihood", 
-     yaxt = "n")#, tck = -0.01)
+     yaxt = "n")
 abline(v = res2$rhat, lty = 1, col = cols[1])
 abline(h = c(max(res2$llik), llikCI, res2$llik[[1]]), lty = 2, col = cols[2])
 abline(v = CI, col = cols[1], lty = 5)
